@@ -17,28 +17,34 @@
 #ifndef __GSWITCHIT_APPLET_H__
 #define __GSWITCHIT_APPLET_H__
 
+#include <gdk/gdkevents.h>
+#include <gdk/gdkkeysyms.h>
+#include <gtk/gtkwidget.h>
 #include <panel-applet.h>
 
 #include "libgswitchit/gswitchit_config.h"
 #include "libgswitchit/gswitchit_plugin_manager.h"
 
 typedef struct _GSwitchItApplet {
-	GSwitchItPluginContainer pluginContainer;
-
-	GSwitchItConfig config;
-	GSwitchItAppletConfig appletConfig;
-	GSwitchItKbdConfig kbdConfig;
-	GSwitchItPluginManager pluginManager;
-
 	GtkWidget *applet;
-	GtkWidget *appletAncestor;
+	GdkWindow *appletAncestor;
 	GtkWidget *notebook;
 	GtkWidget *ebox;
-	GtkWidget *aboutDialog;
-
-	GSList *groupNames;
 } GSwitchItApplet;
 
+typedef struct _GSwitchItAppletGlobals {
+        GSwitchItPluginContainer pluginContainer;
+        GSwitchItPluginManager pluginManager;
+        GSwitchItConfig config;
+        GSwitchItAppletConfig appletConfig;
+        GSwitchItKbdConfig kbdConfig;
+        gchar **groupNames;
+        GSList *appletInstances;
+        GHashTable *previewDialogs;
+	XklEngine *engine;
+	XklConfigRegistry *configRegistry;
+} GSwitchItAppletGlobals;
+
 extern void GSwitchItAppletRevalidate (GSwitchItApplet * sia);
 extern void GSwitchItAppletRevalidateGroup (GSwitchItApplet * sia,
 					    int group);
