@@ -28,7 +28,8 @@
 #include <libgnomeui/gnome-ui-init.h>
 
 #include <libxklavier/xklavier.h>
-#include <libxklavier/xklavier_config.h>
+
+static GSwitchItKbdConfig initialSysKbdConfig;
 
 extern void
 CappletFillActivePluginList (GSwitchItPluginsCapplet * gswic)
@@ -250,19 +251,17 @@ CappletSetup (GSwitchItPluginsCapplet * gswic)
 						      "text", 0,
 						      NULL);
 	GtkTreeSelection *selection;
-	const char *iconFile;
 	glade_gnome_init ();
+
+	gtk_window_set_default_icon_name ("gswitchit-properties-capplet");
+
 	/* default domain! */
-	data = glade_xml_new (GNOME_GLADEDIR "/gswitchit-plugins.glade", "gswitchit_plugins", NULL);	
+	data =
+	    glade_xml_new (GNOME_GLADEDIR "/gswitchit-plugins.glade",
+			   "gswitchit_plugins", NULL);
 	gswic->capplet = capplet =
 	    glade_xml_get_widget (data, "gswitchit_plugins");
-	iconFile = gnome_program_locate_file (NULL,
-					      GNOME_FILE_DOMAIN_PIXMAP,
-					      "gswitchit-properties-capplet.png",
-					      TRUE, NULL);
-	if (iconFile != NULL)
-		gtk_window_set_icon_from_file (GTK_WINDOW (capplet),
-					       iconFile, NULL);
+
 	gtk_object_set_data (GTK_OBJECT (capplet), "gladeData", data);
 	g_signal_connect_swapped (GTK_OBJECT (capplet),
 				  "destroy", G_CALLBACK (g_object_unref),
@@ -316,6 +315,7 @@ int
 main (int argc, char **argv)
 {
 	GSwitchItPluginsCapplet gswic;
+
 	GError *gconf_error = NULL;
 	GConfClient *confClient;
 
@@ -325,7 +325,7 @@ main (int argc, char **argv)
 	memset (&gswic, 0, sizeof (gswic));
 	gnome_program_init ("gswitchit", VERSION,
 			    LIBGNOMEUI_MODULE, argc, argv,
-			    GNOME_PROGRAM_STANDARD_PROPERTIES, NULL); 
+			    GNOME_PROGRAM_STANDARD_PROPERTIES, NULL);
 	if (!gconf_init (argc, argv, &gconf_error)) {
 		g_warning (_("Failed to init GConf: %s\n"),
 			   gconf_error->message);
@@ -333,29 +333,43 @@ main (int argc, char **argv)
 		return 1;
 	}
 	gconf_error = NULL;
-	/*GSwitchItInstallGlibLogAppender(  );*/
-	XklInit (GDK_DISPLAY ());
-	XklConfigInit ();
-	XklConfigLoadRegistry ();
+	/*GSwitchItInstallGlibLogAppender(  ); */
+	gswic.engine = xkl_engine_get_instance (GDK_DISPLAY ());
+	gswic.configRegistry =
+	    xkl_config_registry_get_instance (gswic.engine);
 
 	confClient = gconf_client_get_default ();
 	GSwitchItPluginContainerInit (&gswic.pluginContainer, confClient);
 	g_object_unref (confClient);
 
-	GSwitchItKbdConfigInit (&gswic.kbdConfig, confClient);
-	GSwitchItAppletConfigInit (&gswic.appletConfig, confClient);
+	GSwitchItKbdConfigInit (&gswic.kbdConfig, confClient,
+				gswic.engine);
+	GSwitchItKbdConfigInit (&initialSysKbdConfig, confClient,
+				gswic.engine);
+
+	GSwitchItAppletConfigInit (&gswic.appletConfig, confClient,
+				   gswic.engine);
+
 	GSwitchItPluginManagerInit (&gswic.pluginManager);
-	GSwitchItKbdConfigLoadFromGConf (&gswic.kbdConfig);
+
+	GSwitchItKbdConfigLoadFromXInitial (&initialSysKbdConfig);
+	GSwitchItKbdConfigLoadFromGConf (&gswic.kbdConfig,
+					 &initialSysKbdConfig);
+
 	GSwitchItAppletConfigLoadFromGConf (&gswic.appletConfig);
 	CappletSetup (&gswic);
 	bonobo_main ();
+
 	GSwitchItPluginManagerTerm (&gswic.pluginManager);
+
 	GSwitchItAppletConfigTerm (&gswic.appletConfig);
+
 	GSwitchItKbdConfigTerm (&gswic.kbdConfig);
+	GSwitchItKbdConfigTerm (&initialSysKbdConfig);
+
 	GSwitchItPluginContainerTerm (&gswic.pluginContainer);
-	XklConfigFreeRegistry ();
-	XklConfigTerm ();
-	XklTerm ();
+	g_object_unref (G_OBJECT (gswic.configRegistry));
+	g_object_unref (G_OBJECT (gswic.engine));
 	return 0;
 }
 
@@ -365,8 +379,10 @@ GSwitchItPluginContainerReinitUi (GSwitchItPluginContainer * pc)
 {
 }
 
-GSList *
+gchar **
 GSwitchItPluginLoadLocalizedGroupNames (GSwitchItPluginContainer * pc)
 {
-	return GSwitchItConfigLoadGroupDescriptionsUtf8 (&(((GSwitchItPluginsCapplet *) pc)->config));
+	return
+	    GSwitchItConfigLoadGroupDescriptionsUtf8 (&
+						      (((GSwitchItPluginsCapplet *) pc)->config), (((GSwitchItPluginsCapplet *) pc)->configRegistry));
 }
