@@ -18,13 +18,12 @@
 
 #include <stdio.h>
 #include <string.h>
+#include <stdlib.h>
 #include <X11/keysym.h>
 
+#include <glib/gi18n.h>
 #include <gdk/gdkx.h>
-#include <gnome.h>
-#include <libgnome/gnome-util.h>
-
-#include <libxklavier/xklavier_config.h>
+#include <libgnome/gnome-program.h>
 
 #include "gswitchit_config.h"
 #include "gswitchit_config_private.h"
@@ -36,27 +35,41 @@
 
 #define GSWITCHIT_CONFIG_KEY_PREFIX  KEYBOARD_CONFIG_KEY_PREFIX "/general"
 
-const char GSWITCHIT_CONFIG_DIR[] = GSWITCHIT_CONFIG_KEY_PREFIX;
-const char GSWITCHIT_CONFIG_KEY_DEFAULT_GROUP[] =
+const gchar GSWITCHIT_CONFIG_DIR[] = GSWITCHIT_CONFIG_KEY_PREFIX;
+const gchar GSWITCHIT_CONFIG_KEY_DEFAULT_GROUP[] =
     GSWITCHIT_CONFIG_KEY_PREFIX "/defaultGroup";
-const char GSWITCHIT_CONFIG_KEY_GROUP_PER_WINDOW[] =
+const gchar GSWITCHIT_CONFIG_KEY_GROUP_PER_WINDOW[] =
     GSWITCHIT_CONFIG_KEY_PREFIX "/groupPerWindow";
-const char GSWITCHIT_CONFIG_KEY_HANDLE_INDICATORS[] =
+const gchar GSWITCHIT_CONFIG_KEY_HANDLE_INDICATORS[] =
     GSWITCHIT_CONFIG_KEY_PREFIX "/handleIndicators";
-const char GSWITCHIT_CONFIG_KEY_LAYOUT_NAMES_AS_GROUP_NAMES[]
+const gchar GSWITCHIT_CONFIG_KEY_LAYOUT_NAMES_AS_GROUP_NAMES[]
     = GSWITCHIT_CONFIG_KEY_PREFIX "/layoutNamesAsGroupNames";
 
+#define GSWITCHIT_PREVIEW_CONFIG_KEY_PREFIX  "/apps/gswitchit/preview"
+
+const gchar GSWITCHIT_PREVIEW_CONFIG_DIR[] =
+    GSWITCHIT_PREVIEW_CONFIG_KEY_PREFIX;
+const gchar GSWITCHIT_PREVIEW_CONFIG_KEY_X[] =
+    GSWITCHIT_PREVIEW_CONFIG_KEY_PREFIX "/x";
+const gchar GSWITCHIT_PREVIEW_CONFIG_KEY_Y[] =
+    GSWITCHIT_PREVIEW_CONFIG_KEY_PREFIX "/y";
+const gchar GSWITCHIT_PREVIEW_CONFIG_KEY_WIDTH[] =
+    GSWITCHIT_PREVIEW_CONFIG_KEY_PREFIX "/width";
+const gchar GSWITCHIT_PREVIEW_CONFIG_KEY_HEIGHT[] =
+    GSWITCHIT_PREVIEW_CONFIG_KEY_PREFIX "/height";
+
 /**
  * GSwitchItAppletConfig
  */
 #define GSWITCHIT_APPLET_CONFIG_KEY_PREFIX  "/apps/gswitchit/Applet"
 
-const char GSWITCHIT_APPLET_CONFIG_DIR[] = GSWITCHIT_APPLET_CONFIG_KEY_PREFIX;
-const char GSWITCHIT_APPLET_CONFIG_KEY_SHOW_FLAGS[] =
+const gchar GSWITCHIT_APPLET_CONFIG_DIR[] =
+    GSWITCHIT_APPLET_CONFIG_KEY_PREFIX;
+const gchar GSWITCHIT_APPLET_CONFIG_KEY_SHOW_FLAGS[] =
     GSWITCHIT_APPLET_CONFIG_KEY_PREFIX "/showFlags";
-const char GSWITCHIT_APPLET_CONFIG_KEY_ENABLED_PLUGINS[] =
+const gchar GSWITCHIT_APPLET_CONFIG_KEY_ENABLED_PLUGINS[] =
     GSWITCHIT_APPLET_CONFIG_KEY_PREFIX "/enabledPlugins";
-const char GSWITCHIT_APPLET_CONFIG_KEY_SECONDARIES[] =
+const gchar GSWITCHIT_APPLET_CONFIG_KEY_SECONDARIES[] =
     GSWITCHIT_APPLET_CONFIG_KEY_PREFIX "/secondary";
 
 /**
@@ -64,17 +77,15 @@ const char GSWITCHIT_APPLET_CONFIG_KEY_SECONDARIES[] =
  */
 #define GSWITCHIT_KBD_CONFIG_KEY_PREFIX KEYBOARD_CONFIG_KEY_PREFIX "/kbd"
 
-const char GSWITCHIT_KBD_CONFIG_DIR[] = GSWITCHIT_KBD_CONFIG_KEY_PREFIX;
-const char GSWITCHIT_KBD_CONFIG_KEY_OVERRIDE_SETTINGS[] =
-    GSWITCHIT_KBD_CONFIG_KEY_PREFIX "/overrideSettings";
-const char GSWITCHIT_KBD_CONFIG_KEY_MODEL[] =
+const gchar GSWITCHIT_KBD_CONFIG_DIR[] = GSWITCHIT_KBD_CONFIG_KEY_PREFIX;
+const gchar GSWITCHIT_KBD_CONFIG_KEY_MODEL[] =
     GSWITCHIT_KBD_CONFIG_KEY_PREFIX "/model";
-const char GSWITCHIT_KBD_CONFIG_KEY_LAYOUTS[] =
+const gchar GSWITCHIT_KBD_CONFIG_KEY_LAYOUTS[] =
     GSWITCHIT_KBD_CONFIG_KEY_PREFIX "/layouts";
-const char GSWITCHIT_KBD_CONFIG_KEY_OPTIONS[] =
+const gchar GSWITCHIT_KBD_CONFIG_KEY_OPTIONS[] =
     GSWITCHIT_KBD_CONFIG_KEY_PREFIX "/options";
 
-const char *GSWITCHIT_KBD_CONFIG_ACTIVE[] = {
+const gchar *GSWITCHIT_KBD_CONFIG_ACTIVE[] = {
 	GSWITCHIT_KBD_CONFIG_KEY_MODEL,
 	GSWITCHIT_KBD_CONFIG_KEY_LAYOUTS,
 	GSWITCHIT_KBD_CONFIG_KEY_OPTIONS
@@ -82,16 +93,16 @@ const char *GSWITCHIT_KBD_CONFIG_ACTIVE[] = {
 
 #define GSWITCHIT_KBD_CONFIG_SYSBACKUP_KEY_PREFIX KEYBOARD_CONFIG_KEY_PREFIX "/kbd.sysbackup"
 
-const char GSWITCHIT_KBD_CONFIG_SYSBACKUP_DIR[] =
+const gchar GSWITCHIT_KBD_CONFIG_SYSBACKUP_DIR[] =
     GSWITCHIT_KBD_CONFIG_SYSBACKUP_KEY_PREFIX;
-const char GSWITCHIT_KBD_CONFIG_SYSBACKUP_KEY_MODEL[] =
+const gchar GSWITCHIT_KBD_CONFIG_SYSBACKUP_KEY_MODEL[] =
     GSWITCHIT_KBD_CONFIG_SYSBACKUP_KEY_PREFIX "/model";
-const char GSWITCHIT_KBD_CONFIG_SYSBACKUP_KEY_LAYOUTS[] =
+const gchar GSWITCHIT_KBD_CONFIG_SYSBACKUP_KEY_LAYOUTS[] =
     GSWITCHIT_KBD_CONFIG_SYSBACKUP_KEY_PREFIX "/layouts";
-const char GSWITCHIT_KBD_CONFIG_SYSBACKUP_KEY_OPTIONS[] =
+const gchar GSWITCHIT_KBD_CONFIG_SYSBACKUP_KEY_OPTIONS[] =
     GSWITCHIT_KBD_CONFIG_SYSBACKUP_KEY_PREFIX "/options";
 
-const char *GSWITCHIT_KBD_CONFIG_SYSBACKUP[] = {
+const gchar *GSWITCHIT_KBD_CONFIG_SYSBACKUP[] = {
 	GSWITCHIT_KBD_CONFIG_SYSBACKUP_KEY_MODEL,
 	GSWITCHIT_KBD_CONFIG_SYSBACKUP_KEY_LAYOUTS,
 	GSWITCHIT_KBD_CONFIG_SYSBACKUP_KEY_OPTIONS
@@ -112,12 +123,13 @@ _GSwitchItConfigStringListReset (GSList ** plist)
 }
 
 static Bool
-_GSwitchItKbdConfigGetDescriptions (const char *layoutName,
-				 const char *variantName,
-				 char **layoutShortDescr,
-				 char **layoutDescr,
-				 char **variantShortDescr,
-				 char **variantDescr)
+_GSwitchItKbdConfigGetDescriptions (XklConfigRegistry * configRegistry,
+				    const gchar * layoutName,
+				    const gchar * variantName,
+				    gchar ** layoutShortDescr,
+				    gchar ** layoutDescr,
+				    gchar ** variantShortDescr,
+				    gchar ** variantDescr)
 {
 	static XklConfigItem litem;
 	static XklConfigItem vitem;
@@ -125,8 +137,8 @@ _GSwitchItKbdConfigGetDescriptions (const char *layoutName,
 	layoutName = g_strdup (layoutName);
 
 	g_snprintf (litem.name, sizeof litem.name, "%s", layoutName);
-	if (XklConfigFindLayout (&litem)) {
-		*layoutShortDescr = litem.shortDescription;
+	if (xkl_config_registry_find_layout (configRegistry, &litem)) {
+		*layoutShortDescr = litem.short_description;
 		*layoutDescr = litem.description;
 	} else
 		*layoutShortDescr = *layoutDescr = NULL;
@@ -135,8 +147,9 @@ _GSwitchItKbdConfigGetDescriptions (const char *layoutName,
 		variantName = g_strdup (variantName);
 		g_snprintf (vitem.name, sizeof vitem.name, "%s",
 			    variantName);
-		if (XklConfigFindVariant (layoutName, &vitem)) {
-			*variantShortDescr = vitem.shortDescription;
+		if (xkl_config_registry_find_variant
+		    (configRegistry, layoutName, &vitem)) {
+			*variantShortDescr = vitem.short_description;
 			*variantDescr = vitem.description;
 		} else
 			*variantShortDescr = *variantDescr = NULL;
@@ -155,14 +168,15 @@ _GSwitchItConfigAddListener (GConfClient * confClient,
 			     GConfClientNotifyFunc func,
 			     gpointer userData, int *pid)
 {
-	GError *err = NULL;
-	XklDebug (150, "Listening to [%s]\n", key);
+	GError *gerror = NULL;
+	xkl_debug (150, "Listening to [%s]\n", key);
 	*pid = gconf_client_notify_add (confClient,
-					key, func, userData, NULL, &err);
+					key, func, userData, NULL,
+					&gerror);
 	if (0 == *pid) {
 		g_warning ("Error listening for configuration: [%s]\n",
-			   err->message);
-		g_error_free (err);
+			   gerror->message);
+		g_error_free (gerror);
 	}
 }
 
@@ -178,10 +192,10 @@ _GSwitchItConfigRemoveListener (GConfClient * confClient, int *pid)
 /**
  * extern common functions
  */
-const char *
-GSwitchItKbdConfigMergeItems (const char *parent, const char *child)
+const gchar *
+GSwitchItKbdConfigMergeItems (const gchar * parent, const gchar * child)
 {
-	static char buffer[XKL_MAX_CI_NAME_LENGTH * 2 - 1];
+	static gchar buffer[XKL_MAX_CI_NAME_LENGTH * 2 - 1];
 	*buffer = '\0';
 	if (parent != NULL) {
 		if (strlen (parent) >= XKL_MAX_CI_NAME_LENGTH)
@@ -198,12 +212,13 @@ GSwitchItKbdConfigMergeItems (const char *parent, const char *child)
 }
 
 gboolean
-GSwitchItKbdConfigSplitItems (const char *merged, char **parent, char **child)
+GSwitchItKbdConfigSplitItems (const gchar * merged, gchar ** parent,
+			      gchar ** child)
 {
-	static char pbuffer[XKL_MAX_CI_NAME_LENGTH];
-	static char cbuffer[XKL_MAX_CI_NAME_LENGTH];
+	static gchar pbuffer[XKL_MAX_CI_NAME_LENGTH];
+	static gchar cbuffer[XKL_MAX_CI_NAME_LENGTH];
 	int plen, clen;
-	const char *pos;
+	const gchar *pos;
 	*parent = *child = NULL;
 
 	if (merged == NULL)
@@ -271,25 +286,24 @@ static void
 _GSwitchItKbdConfigCopyFromXklConfig (GSwitchItKbdConfig * kbdConfig,
 				      XklConfigRec * pdata)
 {
-	int i;
 	char **p, **p1;
 	GSwitchItKbdConfigModelSet (kbdConfig, pdata->model);
-	XklDebug (150, "Loaded Kbd model: [%s]\n", pdata->model);
+	xkl_debug (150, "Loaded Kbd model: [%s]\n", pdata->model);
 
 	GSwitchItKbdConfigLayoutsReset (kbdConfig);
 	p = pdata->layouts;
 	p1 = pdata->variants;
-	for (i = pdata->numLayouts; --i >= 0;) {
+	while (p != NULL && *p != NULL) {
 		if (*p1 == NULL || **p1 == '\0') {
-			XklDebug (150, "Loaded Kbd layout: [%s]\n", *p);
+			xkl_debug (150, "Loaded Kbd layout: [%s]\n", *p);
 			_GSwitchItKbdConfigLayoutsAdd (kbdConfig, *p);
 		} else {
 			char fullLayout[XKL_MAX_CI_NAME_LENGTH * 2];
 			g_snprintf (fullLayout, sizeof (fullLayout),
 				    "%s\t%s", *p, *p1);
-			XklDebug (150,
-				  "Loaded Kbd layout with variant: [%s]\n",
-				  fullLayout);
+			xkl_debug (150,
+				   "Loaded Kbd layout with variant: [%s]\n",
+				   fullLayout);
 			_GSwitchItKbdConfigLayoutsAdd (kbdConfig,
 						       fullLayout);
 		}
@@ -299,7 +313,7 @@ _GSwitchItKbdConfigCopyFromXklConfig (GSwitchItKbdConfig * kbdConfig,
 
 	GSwitchItKbdConfigOptionsReset (kbdConfig);
 	p = pdata->options;
-	for (i = pdata->numOptions; --i >= 0;) {
+	while (p != NULL && *p != NULL) {
 		char group[XKL_MAX_CI_NAME_LENGTH];
 		char *option = *p;
 		char *delim =
@@ -309,8 +323,8 @@ _GSwitchItKbdConfigCopyFromXklConfig (GSwitchItKbdConfig * kbdConfig,
 		    ((len = (delim - option)) < XKL_MAX_CI_NAME_LENGTH)) {
 			strncpy (group, option, len);
 			group[len] = 0;
-			XklDebug (150, "Loaded Kbd option: [%s][%s]\n",
-				  group, option);
+			xkl_debug (150, "Loaded Kbd option: [%s][%s]\n",
+				   group, option);
 			GSwitchItKbdConfigOptionsAdd (kbdConfig, group,
 						      option);
 		}
@@ -323,63 +337,69 @@ _GSwitchItKbdConfigCopyToXklConfig (GSwitchItKbdConfig * kbdConfig,
 				    XklConfigRec * pdata)
 {
 	int i;
+	int numLayouts, numOptions;
 	pdata->model =
-	    (kbdConfig->model == NULL) ? NULL : strdup (kbdConfig->model);
+	    (kbdConfig->model ==
+	     NULL) ? NULL : g_strdup (kbdConfig->model);
 
-	pdata->numLayouts = pdata->numVariants =
+	numLayouts =
 	    (kbdConfig->layouts ==
 	     NULL) ? 0 : g_slist_length (kbdConfig->layouts);
-	pdata->numOptions =
+	numOptions =
 	    (kbdConfig->options ==
 	     NULL) ? 0 : g_slist_length (kbdConfig->options);
 
-	XklDebug (150, "Taking %d layouts\n", pdata->numLayouts);
-	if (pdata->numLayouts != 0) {
+	xkl_debug (150, "Taking %d layouts\n", numLayouts);
+	if (numLayouts != 0) {
 		GSList *theLayout = kbdConfig->layouts;
 		char **p1 = pdata->layouts =
-		    calloc ((sizeof (char *)) * pdata->numLayouts, 1);
+		    g_new0 (char *, numLayouts + 1);
 		char **p2 = pdata->variants =
-		    calloc ((sizeof (char *)) * pdata->numVariants, 1);
-		for (i = pdata->numLayouts; --i >= 0;) {
+		    g_new0 (char *, numLayouts + 1);
+		for (i = numLayouts; --i >= 0;) {
 			char *layout, *variant;
 			if (GSwitchItKbdConfigSplitItems
 			    (theLayout->data, &layout, &variant)
 			    && variant != NULL) {
 				*p1 =
 				    (layout ==
-				     NULL) ? NULL : strdup (layout);
+				     NULL) ? g_strdup ("") :
+				    g_strdup (layout);
 				*p2 =
 				    (variant ==
-				     NULL) ? NULL : strdup (variant);
+				     NULL) ? g_strdup ("") :
+				    g_strdup (variant);
 			} else {
 				*p1 =
 				    (theLayout->data ==
-				     NULL) ? NULL : strdup (theLayout->
-							    data);
-				*p2 = NULL;
+				     NULL) ? g_strdup ("") :
+				    g_strdup (theLayout->data);
+				*p2 = g_strdup ("");
 			}
-			XklDebug (150, "Adding [%s]/%p and [%s]/%p\n",
-				  *p1 ? *p1 : "(nil)", *p1,
-				  *p2 ? *p2 : "(nil)", *p2);
+			xkl_debug (150, "Adding [%s]/%p and [%s]/%p\n",
+				   *p1 ? *p1 : "(nil)", *p1,
+				   *p2 ? *p2 : "(nil)", *p2);
 			p1++;
 			p2++;
 			theLayout = theLayout->next;
 		}
 	}
 
-	if (pdata->numOptions != 0) {
+	if (numOptions != 0) {
 		GSList *theOption = kbdConfig->options;
 		char **p = pdata->options =
-		    calloc ((sizeof (char *)) * pdata->numOptions, 1);
-		for (i = pdata->numOptions; --i >= 0;) {
+		    g_new0 (char *, numOptions + 1);
+		for (i = numOptions; --i >= 0;) {
 			char *group, *option;
 			if (GSwitchItKbdConfigSplitItems
 			    (theOption->data, &group, &option)
 			    && option != NULL)
-				*(p++) = strdup (option);
-			else
-				XklDebug (150, "Could not split [%s]\n",
-					  theOption->data);
+				*(p++) = g_strdup (option);
+			else {
+				*(p++) = g_strdup ("");
+				xkl_debug (150, "Could not split [%s]\n",
+					   theOption->data);
+			}
 			theOption = theOption->next;
 		}
 	}
@@ -405,7 +425,7 @@ _GSListStrEqual (GSList * l1, GSList * l2)
 
 static void
 _GSwitchItKbdConfigLoadParams (GSwitchItKbdConfig * kbdConfig,
-			       const char *paramNames[])
+			       const gchar * paramNames[])
 {
 	GError *gerror = NULL;
 	gchar *pc;
@@ -426,7 +446,8 @@ _GSwitchItKbdConfigLoadParams (GSwitchItKbdConfig * kbdConfig,
 		GSwitchItKbdConfigModelSet (kbdConfig, pc);
 		g_free (pc);
 	}
-	XklDebug (150, "Loaded Kbd model: [%s]\n", kbdConfig->model);
+	xkl_debug (150, "Loaded Kbd model: [%s]\n",
+		   kbdConfig->model ? kbdConfig->model : "(null)");
 
 	GSwitchItKbdConfigLayoutsReset (kbdConfig);
 
@@ -443,11 +464,11 @@ _GSwitchItKbdConfigLoadParams (GSwitchItKbdConfig * kbdConfig,
 	}
 
 	while (pl != NULL) {
-		XklDebug (150, "Loaded Kbd layout: [%s]\n", pl->data);
+		xkl_debug (150, "Loaded Kbd layout: [%s]\n", pl->data);
 		_GSwitchItKbdConfigLayoutsAdd (kbdConfig, pl->data);
 		pl = pl->next;
 	}
-	_GSwitchItConfigStringListReset(&pl);
+	_GSwitchItConfigStringListReset (&pl);
 
 	GSwitchItKbdConfigOptionsReset (kbdConfig);
 
@@ -464,53 +485,74 @@ _GSwitchItKbdConfigLoadParams (GSwitchItKbdConfig * kbdConfig,
 	}
 
 	while (pl != NULL) {
-		XklDebug (150, "Loaded Kbd option: [%s]\n", pl->data);
+		xkl_debug (150, "Loaded Kbd option: [%s]\n", pl->data);
 		_GSwitchItKbdConfigOptionsAdd (kbdConfig,
-					       (const char *) pl->data);
+					       (const gchar *) pl->data);
 		pl = pl->next;
 	}
-	_GSwitchItConfigStringListReset(&pl);
+	_GSwitchItConfigStringListReset (&pl);
 }
 
 static void
 _GSwitchItKbdConfigSaveParams (GSwitchItKbdConfig * kbdConfig,
 			       GConfChangeSet * cs,
-			       const char *paramNames[])
+			       const gchar * paramNames[])
 {
 	GSList *pl;
 
-	gconf_change_set_set_string (cs, paramNames[0], kbdConfig->model);
-	XklDebug (150, "Saved Kbd model: [%s]\n", kbdConfig->model);
+	if (kbdConfig->model)
+		gconf_change_set_set_string (cs, paramNames[0],
+					     kbdConfig->model);
+	else
+		gconf_change_set_unset (cs, paramNames[0]);
+	xkl_debug (150, "Saved Kbd model: [%s]\n",
+		   kbdConfig->model ? kbdConfig->model : "(null)");
 
-	pl = kbdConfig->layouts;
-	while (pl != NULL) {
-		XklDebug (150, "Saved Kbd layout: [%s]\n", pl->data);
-		pl = pl->next;
+	if (kbdConfig->layouts) {
+		pl = kbdConfig->layouts;
+		while (pl != NULL) {
+			xkl_debug (150, "Saved Kbd layout: [%s]\n",
+				   pl->data);
+			pl = pl->next;
+		}
+		gconf_change_set_set_list (cs,
+					   paramNames[1],
+					   GCONF_VALUE_STRING,
+					   kbdConfig->layouts);
+	} else {
+		xkl_debug (150, "Saved Kbd layouts: []\n");
+		gconf_change_set_unset (cs, paramNames[1]);
 	}
-	gconf_change_set_set_list (cs,
-				   paramNames[1],
-				   GCONF_VALUE_STRING, kbdConfig->layouts);
-	pl = kbdConfig->options;
-	while (pl != NULL) {
-		XklDebug (150, "Saved Kbd option: [%s]\n", pl->data);
-		pl = pl->next;
+
+	if (kbdConfig->options) {
+		pl = kbdConfig->options;
+		while (pl != NULL) {
+			xkl_debug (150, "Saved Kbd option: [%s]\n",
+				   pl->data);
+			pl = pl->next;
+		}
+		gconf_change_set_set_list (cs,
+					   paramNames[2],
+					   GCONF_VALUE_STRING,
+					   kbdConfig->options);
+	} else {
+		xkl_debug (150, "Saved Kbd options: []\n");
+		gconf_change_set_unset (cs, paramNames[2]);
 	}
-	gconf_change_set_set_list (cs,
-				   paramNames[2],
-				   GCONF_VALUE_STRING, kbdConfig->options);
 }
 
 /**
  * extern GSwitchItConfig config functions
  */
 void
-GSwitchItConfigInit (GSwitchItConfig * config,
-		     GConfClient * confClient)
+GSwitchItConfigInit (GSwitchItConfig * config, GConfClient * confClient,
+		     XklEngine * engine)
 {
 	GError *gerror = NULL;
 
 	memset (config, 0, sizeof (*config));
 	config->confClient = confClient;
+	config->engine = engine;
 	g_object_ref (config->confClient);
 
 	gconf_client_add_dir (config->confClient,
@@ -546,7 +588,7 @@ GSwitchItConfigLoadFromGConf (GSwitchItConfig * config)
 		g_error_free (gerror);
 		gerror = NULL;
 	}
-	XklDebug (150, "groupPerApp: %d\n", config->groupPerApp);
+	xkl_debug (150, "groupPerApp: %d\n", config->groupPerApp);
 
 	config->handleIndicators =
 	    gconf_client_get_bool (config->confClient,
@@ -559,7 +601,8 @@ GSwitchItConfigLoadFromGConf (GSwitchItConfig * config)
 		g_error_free (gerror);
 		gerror = NULL;
 	}
-	XklDebug (150, "handleIndicators: %d\n", config->handleIndicators);
+	xkl_debug (150, "handleIndicators: %d\n",
+		   config->handleIndicators);
 
 	config->layoutNamesAsGroupNames =
 	    gconf_client_get_bool (config->confClient,
@@ -572,7 +615,8 @@ GSwitchItConfigLoadFromGConf (GSwitchItConfig * config)
 		g_error_free (gerror);
 		gerror = NULL;
 	}
-	XklDebug (150, "layoutNamesAsGroupNames: %d\n", config->layoutNamesAsGroupNames);
+	xkl_debug (150, "layoutNamesAsGroupNames: %d\n",
+		   config->layoutNamesAsGroupNames);
 
 	config->defaultGroup =
 	    gconf_client_get_int (config->confClient,
@@ -587,9 +631,10 @@ GSwitchItConfigLoadFromGConf (GSwitchItConfig * config)
 	}
 
 	if (config->defaultGroup < -1
-	    || config->defaultGroup >= XklGetMaxNumGroups ())
+	    || config->defaultGroup >=
+	    xkl_engine_get_max_num_groups (config->engine))
 		config->defaultGroup = -1;
-	XklDebug (150, "defaultGroup: %d\n", config->defaultGroup);
+	xkl_debug (150, "defaultGroup: %d\n", config->defaultGroup);
 }
 
 void
@@ -629,38 +674,40 @@ GSwitchItConfigActivate (GSwitchItConfig * config)
 {
 	gboolean rv = TRUE;
 
-	XklSetGroupPerApp (config->groupPerApp);
-	XklSetIndicatorsHandling (config->handleIndicators);
-	XklSetDefaultGroup (config->defaultGroup);
+	xkl_engine_set_group_per_toplevel_window (config->engine,
+						  config->groupPerApp);
+	xkl_engine_set_indicators_handling (config->engine,
+					    config->handleIndicators);
+	xkl_engine_set_default_group (config->engine,
+				      config->defaultGroup);
 
 	return rv;
 }
 
 void
-GSwitchItConfigLockNextGroup (void)
+GSwitchItConfigLockNextGroup (GSwitchItConfig * config)
 {
-	int group = XklGetNextGroup ();
-	XklLockGroup (group);
+	int group = xkl_engine_get_next_group (config->engine);
+	xkl_engine_lock_group (config->engine, group);
 }
 
 void
-GSwitchIttConfigLockPrevGroup (void)
+GSwitchItConfigLockPrevGroup (GSwitchItConfig * config)
 {
-	int group = XklGetPrevGroup ();
-	XklLockGroup (group);
+	int group = xkl_engine_get_prev_group (config->engine);
+	xkl_engine_lock_group (config->engine, group);
 }
 
 void
-GSwitchItConfigRestoreGroup (void)
+GSwitchItConfigRestoreGroup (GSwitchItConfig * config)
 {
-	int group = XklGetRestoreGroup ();
-	XklLockGroup (group);
+	int group = xkl_engine_get_current_window_group (config->engine);
+	xkl_engine_lock_group (config->engine, group);
 }
 
 void
 GSwitchItConfigStartListen (GSwitchItConfig * config,
-			    GConfClientNotifyFunc func,
-			    gpointer userData)
+			    GConfClientNotifyFunc func, gpointer userData)
 {
 	_GSwitchItConfigAddListener (config->confClient,
 				     GSWITCHIT_CONFIG_DIR, func,
@@ -673,17 +720,19 @@ GSwitchItConfigStopListen (GSwitchItConfig * config)
 	_GSwitchItConfigRemoveListener (config->confClient,
 					&config->configListenerId);
 }
+
 /**
  * extern GSwitchItKbdConfig config functions
  */
 void
 GSwitchItKbdConfigInit (GSwitchItKbdConfig * kbdConfig,
-			GConfClient * confClient)
+			GConfClient * confClient, XklEngine * engine)
 {
 	GError *gerror = NULL;
 
 	memset (kbdConfig, 0, sizeof (*kbdConfig));
 	kbdConfig->confClient = confClient;
+	kbdConfig->engine = engine;
 	g_object_ref (kbdConfig->confClient);
 
 	gconf_client_add_dir (kbdConfig->confClient,
@@ -708,26 +757,39 @@ GSwitchItKbdConfigTerm (GSwitchItKbdConfig * kbdConfig)
 }
 
 void
-GSwitchItKbdConfigLoadFromGConf (GSwitchItKbdConfig * kbdConfig)
+GSwitchItKbdConfigLoadFromGConf (GSwitchItKbdConfig * kbdConfig,
+				 GSwitchItKbdConfig * kbdConfigDefault)
 {
-	GError *gerror = NULL;
-
-	kbdConfig->overrideSettings =
-	    gconf_client_get_bool (kbdConfig->confClient,
-				   GSWITCHIT_KBD_CONFIG_KEY_OVERRIDE_SETTINGS,
-				   &gerror);
-	if (gerror != NULL) {
-		g_warning ("Error reading configuration:%s\n",
-			   gerror->message);
-		kbdConfig->overrideSettings = FALSE;
-		g_error_free (gerror);
-		gerror = NULL;
-	}
-	XklDebug (150, "Loaded Kbd override cmd: [%s]\n",
-		  kbdConfig->overrideSettings ? "true" : "false");
-
 	_GSwitchItKbdConfigLoadParams (kbdConfig,
 				       GSWITCHIT_KBD_CONFIG_ACTIVE);
+
+	if (kbdConfigDefault != NULL) {
+		GSList *pl;
+
+		if (kbdConfig->model == NULL)
+			kbdConfig->model =
+			    g_strdup (kbdConfigDefault->model);
+
+		if (kbdConfig->layouts == NULL) {
+			pl = kbdConfigDefault->layouts;
+			while (pl != NULL) {
+				kbdConfig->layouts =
+				    g_slist_append (kbdConfig->layouts,
+						    g_strdup (pl->data));
+				pl = pl->next;
+			}
+		}
+
+		if (kbdConfig->options == NULL) {
+			pl = kbdConfigDefault->options;
+			while (pl != NULL) {
+				kbdConfig->options =
+				    g_slist_append (kbdConfig->options,
+						    g_strdup (pl->data));
+				pl = pl->next;
+			}
+		}
+	}
 }
 
 void
@@ -740,29 +802,27 @@ GSwitchItKbdConfigLoadFromGConfBackup (GSwitchItKbdConfig * kbdConfig)
 void
 GSwitchItKbdConfigLoadFromXCurrent (GSwitchItKbdConfig * kbdConfig)
 {
-	XklConfigRec data;
-	XklConfigRecInit (&data);
-	if (XklConfigGetFromServer (&data))
-		_GSwitchItKbdConfigCopyFromXklConfig (kbdConfig, &data);
+	XklConfigRec *data = xkl_config_rec_new ();
+	if (xkl_config_rec_get_from_server (data, kbdConfig->engine))
+		_GSwitchItKbdConfigCopyFromXklConfig (kbdConfig, data);
 	else
-		XklDebug (150,
-			  "Could not load keyboard config from server: [%s]\n",
-			  XklGetLastError ());
-	XklConfigRecDestroy (&data);
+		xkl_debug (150,
+			   "Could not load keyboard config from server: [%s]\n",
+			   xkl_get_last_error ());
+	g_object_unref (G_OBJECT (data));
 }
 
 void
 GSwitchItKbdConfigLoadFromXInitial (GSwitchItKbdConfig * kbdConfig)
 {
-	XklConfigRec data;
-	XklConfigRecInit (&data);
-	if (XklConfigGetFromBackup (&data))
-		_GSwitchItKbdConfigCopyFromXklConfig (kbdConfig, &data);
+	XklConfigRec *data = xkl_config_rec_new ();
+	if (xkl_config_rec_get_from_backup (data, kbdConfig->engine))
+		_GSwitchItKbdConfigCopyFromXklConfig (kbdConfig, data);
 	else
-		XklDebug (150,
-			  "Could not load keyboard config from backup: [%s]\n",
-			  XklGetLastError ());
-	XklConfigRecDestroy (&data);
+		xkl_debug (150,
+			   "Could not load keyboard config from backup: [%s]\n",
+			   xkl_get_last_error ());
+	g_object_unref (G_OBJECT (data));
 }
 
 gboolean
@@ -787,11 +847,6 @@ GSwitchItKbdConfigSaveToGConf (GSwitchItKbdConfig * kbdConfig)
 	GError *gerror = NULL;
 
 	cs = gconf_change_set_new ();
-	gconf_change_set_set_bool (cs,
-				   GSWITCHIT_KBD_CONFIG_KEY_OVERRIDE_SETTINGS,
-				   kbdConfig->overrideSettings);
-	XklDebug (150, "Saved Kbd override cmd: [%s]\n",
-		  kbdConfig->overrideSettings ? "true" : "false");
 
 	_GSwitchItKbdConfigSaveParams (kbdConfig, cs,
 				       GSWITCHIT_KBD_CONFIG_ACTIVE);
@@ -836,7 +891,8 @@ GSwitchItKbdConfigModelSet (GSwitchItKbdConfig * kbdConfig,
 	if (kbdConfig->model != NULL)
 		g_free (kbdConfig->model);
 	kbdConfig->model =
-	    (modelName == NULL) ? NULL : g_strdup (modelName);
+	    (modelName == NULL
+	     || modelName[0] == '\0') ? NULL : g_strdup (modelName);
 }
 
 void
@@ -844,7 +900,7 @@ GSwitchItKbdConfigLayoutsAdd (GSwitchItKbdConfig * kbdConfig,
 			      const gchar * layoutName,
 			      const gchar * variantName)
 {
-	const char *merged;
+	const gchar *merged;
 	if (layoutName == NULL)
 		return;
 	merged = GSwitchItKbdConfigMergeItems (layoutName, variantName);
@@ -870,7 +926,7 @@ GSwitchItKbdConfigOptionsAdd (GSwitchItKbdConfig * kbdConfig,
 			      const gchar * groupName,
 			      const gchar * optionName)
 {
-	const char *merged;
+	const gchar *merged;
 	if (groupName == NULL || optionName == NULL)
 		return;
 	merged = GSwitchItKbdConfigMergeItems (groupName, optionName);
@@ -884,7 +940,7 @@ GSwitchItKbdConfigOptionsIsSet (GSwitchItKbdConfig * kbdConfig,
 				const gchar * groupName,
 				const gchar * optionName)
 {
-	const char *merged =
+	const gchar *merged =
 	    GSwitchItKbdConfigMergeItems (groupName, optionName);
 	if (merged == NULL)
 		return FALSE;
@@ -898,14 +954,11 @@ gboolean
 GSwitchItKbdConfigActivate (GSwitchItKbdConfig * kbdConfig)
 {
 	gboolean rv;
+	XklConfigRec *data = xkl_config_rec_new ();
 
-	XklConfigRec data;
-	XklConfigRecInit (&data);
-
-	_GSwitchItKbdConfigCopyToXklConfig (kbdConfig, &data);
-
-	rv = XklConfigActivate (&data);
-	XklConfigRecDestroy (&data);
+	_GSwitchItKbdConfigCopyToXklConfig (kbdConfig, data);
+	rv = xkl_config_rec_activate (data, kbdConfig->engine);
+	g_object_unref (G_OBJECT (data));
 
 	return rv;
 }
@@ -917,7 +970,8 @@ GSwitchItKbdConfigStartListen (GSwitchItKbdConfig * kbdConfig,
 {
 	_GSwitchItConfigAddListener (kbdConfig->confClient,
 				     GSWITCHIT_KBD_CONFIG_DIR, func,
-				     userData, &kbdConfig->configListenerId);
+				     userData,
+				     &kbdConfig->configListenerId);
 }
 
 void
@@ -934,15 +988,15 @@ void
 GSwitchItAppletConfigFreeImages (GSwitchItAppletConfig * appletConfig)
 {
 	GdkPixbuf *pi;
-	GSList* imgNode;
-	while ((imgNode = appletConfig->images) != NULL)
-	{
+	GSList *imgNode;
+	while ((imgNode = appletConfig->images) != NULL) {
 		pi = GDK_PIXBUF (imgNode->data);
 		/* It can be NULL - some images may be missing */
 		if (pi != NULL) {
 			gdk_pixbuf_unref (pi);
 		}
-		appletConfig->images = g_slist_remove_link (appletConfig->images, imgNode);
+		appletConfig->images =
+		    g_slist_remove_link (appletConfig->images, imgNode);
 		g_slist_free_1 (imgNode);
 	}
 }
@@ -966,7 +1020,8 @@ GSwitchItAppletConfigGetImagesFile (GSwitchItAppletConfig *
 
 		if (fullLayoutName != NULL) {
 			char *l, *v;
-			GSwitchItKbdConfigSplitItems (fullLayoutName, &l, &v);
+			GSwitchItKbdConfigSplitItems (fullLayoutName, &l,
+						      &v);
 			if (l != NULL) {
 				// probably there is something in theme?
 				iconInfo = gtk_icon_theme_lookup_icon
@@ -977,7 +1032,7 @@ GSwitchItAppletConfigGetImagesFile (GSwitchItAppletConfig *
 	// fallback to the default value
 	if (iconInfo == NULL) {
 		iconInfo = gtk_icon_theme_lookup_icon
-		    (appletConfig->iconTheme, "applet-error", 48, 0);
+		    (appletConfig->iconTheme, "stock_dialog-error", 48, 0);
 	}
 	if (iconInfo != NULL) {
 		imageFile =
@@ -998,53 +1053,62 @@ GSwitchItAppletConfigLoadImages (GSwitchItAppletConfig * appletConfig,
 	if (!appletConfig->showFlags)
 		return;
 
-	for (i = XklGetMaxNumGroups (); --i >= 0;) {
+	for (i = xkl_engine_get_max_num_groups (appletConfig->engine);
+	     --i >= 0;) {
 		GdkPixbuf *image = NULL;
 		char *imageFile =
 		    GSwitchItAppletConfigGetImagesFile (appletConfig,
 							kbdConfig, i);
 
 		if (imageFile != NULL) {
-			GError *err = NULL;
+			GError *gerror = NULL;
 			image =
-			    gdk_pixbuf_new_from_file (imageFile, &err);
+			    gdk_pixbuf_new_from_file (imageFile, &gerror);
 			if (image == NULL) {
-				GtkWidget *dialog = gtk_message_dialog_new (NULL, 
-									    GTK_DIALOG_DESTROY_WITH_PARENT,
-									    GTK_MESSAGE_ERROR,
-									    GTK_BUTTONS_OK,
-									    _("There was an error loading an image: %s"),
-									    err->message);
+				GtkWidget *dialog =
+				    gtk_message_dialog_new (NULL,
+							    GTK_DIALOG_DESTROY_WITH_PARENT,
+							    GTK_MESSAGE_ERROR,
+							    GTK_BUTTONS_OK,
+							    _
+							    ("There was an error loading an image: %s"),
+							    gerror->
+							    message);
 				g_signal_connect (G_OBJECT (dialog),
 						  "response",
-						  G_CALLBACK (gtk_widget_destroy), NULL);
+						  G_CALLBACK
+						  (gtk_widget_destroy),
+						  NULL);
 
-				gtk_window_set_resizable (GTK_WINDOW (dialog), FALSE);
+				gtk_window_set_resizable (GTK_WINDOW
+							  (dialog), FALSE);
 
 				gtk_widget_show (dialog);
-				g_error_free (err);
+				g_error_free (gerror);
 			}
-			XklDebug (150,
-				  "Image %d[%s] loaded -> %p[%dx%d]\n",
-				  i, imageFile, image,
-				  gdk_pixbuf_get_width (image),
-				  gdk_pixbuf_get_height (image));
+			xkl_debug (150,
+				   "Image %d[%s] loaded -> %p[%dx%d]\n",
+				   i, imageFile, image,
+				   gdk_pixbuf_get_width (image),
+				   gdk_pixbuf_get_height (image));
 			g_free (imageFile);
 		}
 		/* We append the image anyway - even if it is NULL! */
-		appletConfig->images = g_slist_prepend (appletConfig->images, image);
+		appletConfig->images =
+		    g_slist_prepend (appletConfig->images, image);
 	}
 }
 
 void
 GSwitchItAppletConfigInit (GSwitchItAppletConfig * appletConfig,
-			   GConfClient * confClient)
+			   GConfClient * confClient, XklEngine * engine)
 {
 	GError *gerror = NULL;
 	gchar *sp, *datadir;
 
 	memset (appletConfig, 0, sizeof (*appletConfig));
 	appletConfig->confClient = confClient;
+	appletConfig->engine = engine;
 	g_object_ref (appletConfig->confClient);
 
 	gconf_client_add_dir (appletConfig->confClient,
@@ -1184,7 +1248,9 @@ GSwitchItAppletConfigSaveToGConf (GSwitchItAppletConfig * appletConfig)
 void
 GSwitchItAppletConfigActivate (GSwitchItAppletConfig * appletConfig)
 {
-	XklSetSecondaryGroupsMask (appletConfig->secondaryGroupsMask);
+	xkl_engine_set_secondary_groups_mask (appletConfig->engine,
+					      appletConfig->
+					      secondaryGroupsMask);
 }
 
 void
@@ -1194,7 +1260,8 @@ GSwitchItAppletConfigStartListen (GSwitchItAppletConfig * appletConfig,
 {
 	_GSwitchItConfigAddListener (appletConfig->confClient,
 				     GSWITCHIT_APPLET_CONFIG_DIR, func,
-				     userData, &appletConfig->configListenerId);
+				     userData,
+				     &appletConfig->configListenerId);
 }
 
 void
@@ -1205,27 +1272,31 @@ GSwitchItAppletConfigStopListen (GSwitchItAppletConfig * appletConfig)
 }
 
 Bool
-GSwitchItKbdConfigGetDescriptions (const char *name,
-				char **layoutShortDescr,
-				char **layoutDescr,
-				char **variantShortDescr,
-				char **variantDescr)
+GSwitchItKbdConfigGetDescriptions (XklConfigRegistry * configRegistry,
+				   const gchar * name,
+				   gchar ** layoutShortDescr,
+				   gchar ** layoutDescr,
+				   gchar ** variantShortDescr,
+				   gchar ** variantDescr)
 {
 	char *layoutName = NULL, *variantName = NULL;
-	if (!GSwitchItKbdConfigSplitItems (name, &layoutName, &variantName))
+	if (!GSwitchItKbdConfigSplitItems
+	    (name, &layoutName, &variantName))
 		return FALSE;
-	return _GSwitchItKbdConfigGetDescriptions (layoutName, variantName,
-						layoutShortDescr,
-						layoutDescr,
-						variantShortDescr,
-						variantDescr);
+	return _GSwitchItKbdConfigGetDescriptions (configRegistry,
+						   layoutName,
+						   variantName,
+						   layoutShortDescr,
+						   layoutDescr,
+						   variantShortDescr,
+						   variantDescr);
 }
 
-const char *
-GSwitchItKbdConfigFormatFullLayout (const char *layoutDescr,
-				 const char *variantDescr)
+const gchar *
+GSwitchItKbdConfigFormatFullLayout (const gchar * layoutDescr,
+				    const gchar * variantDescr)
 {
-	static char fullDescr[XKL_MAX_CI_DESC_LENGTH * 2];
+	static gchar fullDescr[XKL_MAX_CI_DESC_LENGTH * 2];
 	if (variantDescr == NULL)
 		g_snprintf (fullDescr, sizeof (fullDescr), "%s",
 			    layoutDescr);
@@ -1235,55 +1306,221 @@ GSwitchItKbdConfigFormatFullLayout (const char *layoutDescr,
 	return fullDescr;
 }
 
-GSList *
-GSwitchItConfigLoadGroupDescriptionsUtf8 (GSwitchItConfig * config)
+gchar **
+GSwitchItConfigLoadGroupDescriptionsUtf8 (GSwitchItConfig * config,
+					  XklConfigRegistry *
+					  configRegistry)
 {
 	int i;
-	const char **pNativeNames = XklGetGroupNames ();
-	GSList *rv = NULL;
-	if ((XklGetBackendFeatures() & XKLF_MULTIPLE_LAYOUTS_SUPPORTED) && 
-	    config->layoutNamesAsGroupNames) {
-		XklConfigRec xklConfig;
-		XklConfigRecInit (&xklConfig);
-		if (XklConfigGetFromServer (&xklConfig)) {
-			char **pl = xklConfig.layouts;
-			char **pv = xklConfig.variants;
-			for (i = xklConfig.numLayouts; --i >= 0;) {
+	const gchar **pNativeNames =
+	    xkl_engine_get_groups_names (config->engine);
+	guint totalGroups = xkl_engine_get_num_groups (config->engine);
+	guint totalLayouts;
+	gchar **rv = g_new0 (char *, totalGroups + 1);
+	gchar **pCurrentDescr = rv;
+
+	if ((xkl_engine_get_features (config->engine) &
+	     XKLF_MULTIPLE_LAYOUTS_SUPPORTED)
+	    && config->layoutNamesAsGroupNames) {
+		XklConfigRec *xklConfig = xkl_config_rec_new ();
+		if (xkl_config_rec_get_from_server
+		    (xklConfig, config->engine)) {
+			char **pl = xklConfig->layouts;
+			char **pv = xklConfig->variants;
+			i = totalGroups;
+			while (pl != NULL && *pl != NULL && i >= 0) {
 				char *lSDescr;
 				char *lDescr;
 				char *vSDescr;
 				char *vDescr;
 				if (_GSwitchItKbdConfigGetDescriptions
-				    (*pl++, *pv++, &lSDescr, &lDescr,
-				     &vSDescr, &vDescr)) {
+				    (configRegistry, *pl++, *pv++,
+				     &lSDescr, &lDescr, &vSDescr,
+				     &vDescr)) {
 					char *nameUtf =
 					    g_locale_to_utf8
 					    (GSwitchItKbdConfigFormatFullLayout
 					     (lDescr, vDescr), -1, NULL,
 					     NULL, NULL);
-					rv = g_slist_append (rv, nameUtf);
+					*pCurrentDescr++ = nameUtf;
 				} else {
-					rv = g_slist_append (rv, g_strdup(""));
+					*pCurrentDescr++ = g_strdup ("");
 				}
 			}
 		}
-		XklConfigRecDestroy (&xklConfig);
+		g_object_unref (G_OBJECT (xklConfig));
 		/* Worst case - multiple layous - but SOME of them are multigrouped :((( 
 		   We cannot do much - just add empty descriptions. 
 		   The UI is going to be messy. 
 		   Canadian layouts are famous for this sh.t. */
-		if (g_slist_length (rv) != XklGetNumGroups())
-		{
-			XklDebug (0, "The mismatch between "
-				  "the number of groups: %d and number of layouts: %d\n",
-				  XklGetNumGroups(), g_slist_length (rv));
-			for (i = XklGetNumGroups() - g_slist_length (rv); --i>=0;)
-				rv = g_slist_append (rv, g_strdup(""));
+		totalLayouts = g_strv_length (rv);
+		if (totalLayouts != totalGroups) {
+			xkl_debug (0,
+				   "The mismatch between "
+				   "the number of groups: %d and number of layouts: %d\n",
+				   totalGroups, totalLayouts);
+			pCurrentDescr = rv + totalLayouts;
+			for (i = totalGroups - totalLayouts; --i >= 0;)
+				*pCurrentDescr++ = g_strdup ("");
 		}
 	}
-	if (rv == NULL)
-		for (i = XklGetNumGroups (); --i >= 0;)
-			rv = g_slist_append (rv, g_strdup (*pNativeNames++));
+	totalLayouts = g_strv_length (rv);
+	if (!totalLayouts)
+		for (i = totalGroups; --i >= 0;)
+			*pCurrentDescr++ = g_strdup (*pNativeNames++);
 
 	return rv;
 }
+
+gchar *
+GSwitchItKbdConfigToString (const GSwitchItKbdConfig * config)
+{
+	gchar *layouts = NULL, *options = NULL;
+	GString *buffer = g_string_new (NULL);
+
+	GSList *iter;
+	gint count;
+	gchar *result;
+
+	if (config->layouts) {
+		/* g_slist_length is "expensive", so we determinate the length on the fly */
+		for (iter = config->layouts, count = 0; iter;
+		     iter = iter->next, ++count) {
+			if (buffer->len)
+				g_string_append (buffer, " ");
+
+			g_string_append (buffer,
+					 (const gchar *) iter->data);
+		}
+
+		layouts =
+		    g_strdup_printf (ngettext
+				     ("layout \"%s\"", "layouts \"%s\"",
+				      count), buffer->str);
+		g_string_truncate (buffer, 0);
+	}
+	if (config->options) {
+		/* g_slist_length is "expensive", so we determinate the length on the fly */
+		for (iter = config->options, count = 0; iter;
+		     iter = iter->next, ++count) {
+			if (buffer->len)
+				g_string_append (buffer, " ");
+
+			g_string_append (buffer,
+					 (const gchar *) iter->data);
+		}
+
+		options =
+		    g_strdup_printf (ngettext
+				     ("option \"%s\"", "options \"%s\"",
+				      count), buffer->str);
+		g_string_truncate (buffer, 0);
+	}
+
+	g_string_free (buffer, TRUE);
+
+	result =
+	    g_strdup_printf (_("model \"%s\", %s and %s"), config->model,
+			     layouts ? layouts : _("no layout"),
+			     options ? options : _("no options"));
+
+	g_free (options);
+	g_free (layouts);
+
+	return result;
+}
+
+GdkRectangle *
+GSwitchItPreviewLoad (void)
+{
+	GError *gerror = NULL;
+	GdkRectangle *rv = NULL;
+	gint x, y, w, h;
+	GConfClient *confClient = gconf_client_get_default ();
+
+	if (confClient == NULL)
+		return NULL;
+
+	x = gconf_client_get_int (confClient,
+				  GSWITCHIT_PREVIEW_CONFIG_KEY_X, &gerror);
+	if (gerror != NULL) {
+		xkl_debug (0, "Error getting the preview x: %s\n",
+			   gerror->message);
+		g_error_free (gerror);
+		g_object_unref (G_OBJECT (confClient));
+		return NULL;
+	}
+
+	y = gconf_client_get_int (confClient,
+				  GSWITCHIT_PREVIEW_CONFIG_KEY_Y, &gerror);
+	if (gerror != NULL) {
+		xkl_debug (0, "Error getting the preview y: %s\n",
+			   gerror->message);
+		g_error_free (gerror);
+		g_object_unref (G_OBJECT (confClient));
+		return NULL;
+	}
+
+	w = gconf_client_get_int (confClient,
+				  GSWITCHIT_PREVIEW_CONFIG_KEY_WIDTH,
+				  &gerror);
+	if (gerror != NULL) {
+		xkl_debug (0, "Error getting the preview width: %s\n",
+			   gerror->message);
+		g_error_free (gerror);
+		g_object_unref (G_OBJECT (confClient));
+		return NULL;
+	}
+
+	h = gconf_client_get_int (confClient,
+				  GSWITCHIT_PREVIEW_CONFIG_KEY_HEIGHT,
+				  &gerror);
+	if (gerror != NULL) {
+		xkl_debug (0, "Error getting the preview height: %s\n",
+			   gerror->message);
+		g_error_free (gerror);
+		g_object_unref (G_OBJECT (confClient));
+		return NULL;
+	}
+
+	g_object_unref (G_OBJECT (confClient));
+
+	// default values should be just ignored
+	if (x == -1 || y == -1 || w == -1 || h == -1)
+		return NULL;
+
+	rv = g_new (GdkRectangle, 1);
+	rv->x = x;
+	rv->y = y;
+	rv->width = w;
+	rv->height = h;
+	return rv;
+}
+
+void
+GSwitchItPreviewSave (GdkRectangle * rect)
+{
+	GConfClient *confClient = gconf_client_get_default ();
+	GConfChangeSet *cs;
+	GError *gerror = NULL;
+
+	cs = gconf_change_set_new ();
+
+	gconf_change_set_set_int (cs, GSWITCHIT_PREVIEW_CONFIG_KEY_X,
+				  rect->x);
+	gconf_change_set_set_int (cs, GSWITCHIT_PREVIEW_CONFIG_KEY_Y,
+				  rect->y);
+	gconf_change_set_set_int (cs, GSWITCHIT_PREVIEW_CONFIG_KEY_WIDTH,
+				  rect->width);
+	gconf_change_set_set_int (cs, GSWITCHIT_PREVIEW_CONFIG_KEY_HEIGHT,
+				  rect->height);
+
+	gconf_client_commit_change_set (confClient, cs, TRUE, &gerror);
+	if (gerror != NULL) {
+		g_warning ("Error saving preview configuration: %s\n",
+			   gerror->message);
+		g_error_free (gerror);
+	}
+	gconf_change_set_unref (cs);
+	g_object_unref (G_OBJECT (confClient));
+}
